#clear the workspace
rm(list = ls())

#adhere to the sum-to-zero convention for effect weights
options(contrasts=c("contr.sum","contr.poly"))

# set the working directory to the current directory with all the scripts
setwd("C:/Users/User/Documents/R")

# read the data from a csv file
statsurvey<-read.table("Study5S-complete-data-forR-long.csv",header=TRUE,sep=",")

# show the variable names and the first few rows of values
head(statsurvey)

# focus on the outcome uncertainty condition. Change the "u" to "v" to look at the amount uncertainty condition
statsurvey <- statsurvey[ which(statsurvey$uncertainDiscoType == "v"), ]

# show the variable names and the first few rows of values
head(statsurvey)

#for conveniece and clarity, create new variables called X, Y, and M, plus covariate C
statsurvey$X <- statsurvey$uncertainDiscoTime
statsurvey$Y <- statsurvey$choiceShift
statsurvey$M <- statsurvey$complexityShift
statsurvey$C <- statsurvey$sign

# add the data object to the searchlist
attach(statsurvey)

library(boot) 					#Accesses the R package called "boot" that contains
								#specialized bootstrapping functions.
		
#Function to calculate a, b, ab, c, c'
abfun <- function(data, d)  {
  E<- data[d,]				# allows boot to select sample
  fit.0 <- lm(Y  ~ X + C, data=E)		#regression to estimate c
  fit.1 <- lm(M  ~ X + C, data=E) 		#regression to estimate a
  fit.2 <- lm(Y ~ X + M + C, data=E)	#regression to estimate b, c'
  ccoef<-coef(fit.0)				#pull coef c from regression output
  acoef<-coef(fit.1)				#pull coef a from regression output
  bcoef<-coef(fit.2)				#pull coefs b and c' from regression output
  ab<-acoef[2]*bcoef[3]			#calc ab
  med<-c(acoef[2], bcoef[3], ab, ccoef[2], bcoef[2]) #create vector: a,b,ab,c',c
  return(med)					#output med to the boot function
}

# bootstrapping with R=1000 replications; only use R=100 when testing the script; then, when you have it working properly, use R=10000
abboots <- boot(data=statsurvey, statistic=abfun,
   R=10000)

#Print the path coefficients from the original sample
Coefficient<-c("a","b","ab","c","c'") 
NormalEsts<- data.frame(Coefficient, abboots$t0)
names(NormalEsts)[c(2)] <- c("Value")
NormalEsts

# Get a sequence of confidence intervals for ab, from 50% to 99.9%; NOTE: R calculates an ever-widening sequence of confidence intervals. At some point in the sequence an interval will include zero. To find out the obtained p-value: (1) find the first confidence interval in the sequence that includes zero; (2) count back one interval; (3) subtract that interval's % from 100%.  Example: If the first confidence interval to include zero is the 98% interval, then the obtained p-value is 100-97=3% or p=.03.
cl<-c(0.5,0.6,0.7,0.8,0.9,0.91,0.92,0.93,0.94,0.95,0.96,0.97,0.98,0.99,0.991,0.992,0.993,0.994, 0.995,0.996,0.997,0.998, 0.999)
abci<-boot.ci(abboots, conf=cl, type=c("perc","bca"), index=3)  #Calc Percentile and BCA CIs for ab
abci														#Show the output of boot.ci

# Create filled density plots of a,b,a*b,c,c'
den1 <- density(abboots$t[,1])
den2 <- density(abboots$t[,2])
den3 <- density(abboots$t[,3])
den4 <- density(abboots$t[,4])
den5 <- density(abboots$t[,5])

# 5 figures arranged in 3 rows and 3 columns
layout(matrix(c(1,2,3,3,4,5), 3, 2, byrow = TRUE))
plot(den1, main="a")
polygon(den1, col="red", border="blue")
plot(den2, main="b")
polygon(den2, col="red", border="blue")
plot(den3, main="ab")
polygon(den3, col="red", border="blue")
plot(den4, main="c")
polygon(den4, col="red", border="blue")
plot(den5, main="c'")
polygon(den5, col="red", border="blue")
mtext("Kernel Density Plots", side=3, outer=TRUE, line=-2)

